// Ycon -- utility to translate message dropped into window
// into list-structured text.
// 2008/07/27
// Open Source
// Copyright 2008 Peter J. Goodeve

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <string.h>
#include <Application.h>
#include <Window.h>
#include <Entry.h>
#include <Path.h>
#include <Font.h>
#include <ByteOrder.h>
#include <Clipboard.h>

//#define DPRINTF(str) printf str
#define DPRINTF(str) 

#define ZDPRINTF(str) 

/////////////////////////////////////////////////////////////////


class WMmsgWindow : public BWindow 
{
public:
	WMmsgWindow(BRect frame, window_feel mode, char *title="Ycon"); 
	bool QuitRequested();
	void MessageReceived(BMessage *msg);
};

WMmsgWindow::WMmsgWindow(BRect frame, window_feel mode, char *title)
				: BWindow(frame, title,
					 B_TITLED_WINDOW_LOOK, mode, 0 )
{}

bool WMmsgWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return true;
}

bool pass_data = false;

void showMsg(BMessage *msg) {
	char  *name; 
	uint32  type; 
	int32   count;
	uint32 code;
	code = htonl(msg->what);
	printf("['%.4s',\n", &code);
	for(int32 i = 0; 
		msg->GetInfo(B_ANY_TYPE, i, &name, &type, &count) == B_OK; i++ ) {
		code = htonl(type);
		printf("  ['%s', '%.4s', \n", name, &code);
		const void *data;
		ssize_t numbytes;
		for (int32 index=0; index<count; index++) {
			msg->FindData(name, type, index, &data, &numbytes);
			switch (type) {
			case 'CSTR':
				printf("    '%s',\n", data);
				break;
			case 'MIME':	// no terminating null!
				printf("    '%.*s',\n", numbytes, data);
				break;
			case 'LONG':
				printf("    %d,\n", *(int32 *)data);
				break;
			case 'PNTR':
				printf("    0x%x,\n", *(int32 *)data);
				break;
			case 'RREF': {
					entry_ref r;
					msg->FindRef(name, index, &r);
					BEntry ent(&r);
					BPath path(&ent);
					printf("    ['%s','%s'],\n",r.name, path.Path());
				}
				break;
			case 'BPNT': {
					BPoint p;
					msg->FindPoint(name, index, &p);
					printf("    [%.1f,%.1f],\n",p.x, p.y);
				}
				break;
			default:
				if (pass_data) {	// output the actual bytes in hex
					printf("    '");
					uint8 *dp = (uint8 *)data;
					for (int i=0; i<numbytes; i++)
						printf("\\%o", *dp++);
					printf("',\n");
				} else
					printf("    '%d bytes',\n", numbytes);
				break;
			}
		}
		printf("  ],\n");
	}
	printf("]\n");
	fflush(stdout);
}

void WMmsgWindow::MessageReceived(BMessage *msg) {
	switch(msg->what) {
		case B_SIMPLE_DATA:
		case B_MIME_DATA:
			showMsg(msg);
			break;
		case B_PASTE:
			if (be_clipboard->Lock()) {
				showMsg(be_clipboard->Data());
				be_clipboard->Unlock();
			}
			break;
	}
}



void showWindow(int x=20, int y=20,
		 window_feel mode=B_NORMAL_WINDOW_FEEL, char *title="Ycon")
{
	WMmsgWindow		*aWindow;
	BRect			aRect;

	float twidth = be_bold_font->StringWidth(title);
	aRect.Set(x, y, x+twidth+60, y+30);
	aWindow = new WMmsgWindow(aRect, mode, title);
			
	aWindow->Show();
}


//////////////////////////////////////////////////////

class YconApplication : public BApplication {
public:
	YconApplication(const char *sig) : BApplication(sig),
		mode(B_NORMAL_WINDOW_FEEL), xpos(20), ypos(20),
		title("Ycon") {}
	void ArgvReceived(int32 argc, char **argv) {
		for (int i=1; i<argc; i++) {	// can't be bothered with getopt for this
			char *arg = argv[i];
			if (*arg++ != '-' || !*arg) break;	// quietly ignore garbage
			if (*arg == 'f') mode = B_FLOATING_ALL_WINDOW_FEEL;
			else if (*arg == 'd') pass_data = true;
			else if (*arg == 'x') {
				if (!*++arg) arg = argv[++i];
				xpos = atoi(arg);
			}
			else if (*arg == 'y') {
				if (!*++arg) arg = argv[++i];
				ypos = atoi(arg);
			}
			else if (*arg == 't') {
				if (!*++arg) arg = argv[++i];
				title = arg;
			}
			if (i >= argc) {	// we overflowed somehow
				fprintf(stderr, "bad arguments\n");
				xpos = ypos = 20;
				title = "Ycon";
				break;
			}
		}
	}
	void ReadyToRun()
	{
		showWindow(xpos, ypos, mode, title);
	}
private:
	window_feel mode;
	int xpos, ypos;
	char *title;
};

//////////////////////////////////////////////////////

int main(int32 argc, char**argv)
{	
	YconApplication *myApplication;
	myApplication = new YconApplication("application/x-vnd.goodeve-ycon");
	myApplication->Run();
	delete(myApplication);
	return(0);
}

